#include "interface.hpp"
#include <algorithm>
#include <iostream>
#include <string>
#include <vector>

Interface::Interface(void)
{
    currentType = INSERTION;
    allLists = 0;
    sortGiven = 0;
    return;
}

// Checks for command line arguments
void Interface::CheckArguments(int argc, char* arguments[])
{
    std::string tempStr;
    for (int i = 0; i < argc; i++)
    {
        tempStr = arguments[i];
        if ((tempStr == "-a") || (tempStr == "--all"))
            allLists = 1;
        if ((tempStr == "-f") || (tempStr == "--filename"))
            benchmark.SetFilename(arguments[i + 1]);
        if ((tempStr == "-d") || (tempStr == "--default"))
            benchmark.SetFilename("test/PERM/perm15K.txt");
        if ((tempStr == "-o") || (tempStr == "--output"))
            benchmark.SetOutput(arguments[i + 1]);
        if ((tempStr == "-s") || (tempStr == "--sort-type"))
        {
            sortGiven = 1;
            tempStr = arguments[i + 1];
            if (tempStr == "bst")
                currentType = BST;
            if (tempStr == "rbt")
                currentType = RBT;
            if (tempStr == "insertion")
                currentType = INSERTION;
            if (tempStr == "merge")
                currentType = MERGE;
            if (tempStr == "heap")
                currentType = HEAP;
            if (tempStr == "all")
                sortGiven = 0;
        }
    }
    return;
}

void Interface::Construction(void)
{
    CheckRequirements();
    benchmark.ReadWordFile();
    if (currentType == BST)
        benchmark.ConstructBST();
    if (currentType == RBT)
        benchmark.ConstructRBT();
}

bool Interface::IsBST(void)
{
    if (currentType == BST) return true;
    return false;
}

bool Interface::IsOnlySorting(void)
{
    if ((currentType == INSERTION) || (currentType == MERGE) || (currentType == HEAP))
        return true;
    return false;
}

void Interface::Sort(void)
{
    ;
}

bool Interface::UserInput(void)
{
    PrintOptions();
    std::cin >> userInput;
    if (!userInput) return 0;
    std::cout << "Enter word you wish to find: ";
    std::cin >> givenWord;
    std::transform(givenWord.begin(), givenWord.end(), givenWord.begin(), ::toupper);
    std::cout << std::endl;
    return 1;
}


void Interface::BSTRequestExecution(void)
{
    switch (userInput)
    {
        case 1:
            // Insert
            benchmark.BSTInsert(givenWord);
            break;
        case 2:
            // Search
            benchmark.BSTSearch(givenWord);
            break;
        case 3:
            // InOrderTreeTraversal
            benchmark.BSTInOrderTreeTraversal(givenWord);
            break;
        case 4:
            // PrintParentKey
            benchmark.BSTPrintParentKey(givenWord);
            break;
        case 5:
            // PrintLeftChild
            benchmark.BSTPrintLeftChild(givenWord);
            break;
        case 6:
            // PrintRightChild
            benchmark.BSTPrintRightChild(givenWord);
            break;
        case 7:
            // PrintPathToRoot
            benchmark.BSTPrintPathToRoot(givenWord);
            break;
        case 8:
            // PrintColor
            std::cout << "This method is not available for BST" << std::endl;
            break;
        case 9:
            // PrintParentColor
            std::cout << "This method is not available for BST" << std::endl;
            break;
        case 10:
            // PrintUncleColor
            std::cout << "This method is not available for BST" << std::endl;
            break;
        default:
            std::cout << "Please enter a valid option." << std::endl;
            std::cout << std::endl << std::endl << std::endl;
            break;
    }
}

void Interface::RBTRequestExecution(void)
{
    switch (userInput)
    {
        case 1:
            // Insert
            benchmark.RBTInsert(givenWord);
            break;
        case 2:
            // Search
            benchmark.RBTSearch(givenWord);
            break;
        case 3:
            // InOrderTreeTraversal
            benchmark.RBTInOrderTreeTraversal(givenWord);
            break;
        case 4:
            // PrintParentKey
            benchmark.RBTPrintParentKey(givenWord);
            break;
        case 5:
            // PrintLeftChild
            benchmark.RBTPrintLeftChild(givenWord);
            break;
        case 6:
            // PrintRightChild
            benchmark.RBTPrintRightChild(givenWord);
            break;
        case 7:
            // PrintPathToRoot
            benchmark.RBTPrintPathToRoot(givenWord);
            break;
        case 8:
            // PrintColor
            benchmark.RBTPrintColor(givenWord);
            break;
        case 9:
            // PrintParentColor
            benchmark.RBTPrintParentColor(givenWord);
            break;
        case 10:
            // PrintUncleColor
            benchmark.RBTPrintUncleColor(givenWord);
            break;
        default:
            std::cout << "Please enter a valid option." << std::endl;
            std::cout << std::endl << std::endl << std::endl;
            break;
    }
}

void Interface::CheckRequirements(void)
{
    if (!sortGiven)
    {
        std::cout << "Please enter a sorting type to use." << std::endl;
        exit(1);
    }
    return;
}

void Interface::PrintOptions(void)
{
    std::cout << "General Options:" << std::endl;
    std::cout << "\t1) Insert" << std::endl;
    std::cout << "\t2) Search" << std::endl;
    std::cout << "\t3) InOrderTreeTraversal" << std::endl;
    std::cout << "\t4) PrintParentKey" << std::endl;
    std::cout << "\t5) PrintLeftChild" << std::endl;
    std::cout << "\t6) PrintRightChild" << std::endl;
    std::cout << "\t7) PrintPathToRoot" << std::endl;
    std::cout << "Options for RBT:" << std::endl;
    std::cout << "\t8) PrintColor" << std::endl;
    std::cout << "\t9) PrintParentColor" << std::endl;
    std::cout << "\t10) PrintUncleColor" << std::endl;
    std::cout << "\t0) Exit" << std::endl;
    std::cout << "Enter the number of the option you wish to use: ";
    return;
}

